package com.wyattjmiller.audiomanv3

import android.content.Intent
import android.content.SharedPreferences
import android.graphics.Color
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.preference.PreferenceManager
import android.view.*
import android.widget.TextView
import android.widget.Toast
import androidx.fragment.app.DialogFragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView

class MainActivity : AppCompatActivity(), FoodDialogFragment.onCourseEnteredListener {
    private var mFoodListRecyclerView: RecyclerView? = null
    private var mFoodAdapter: FoodAdapter? = null
    private lateinit var mFoodDb: FoodDatabase
    private lateinit var mSelectedFood: Food
    private var mActionMode: ActionMode? = null
    private var mSelectedFoodPosition = RecyclerView.NO_POSITION
    private lateinit var mDialog: FoodDialogFragment
    private var mSharedPrefs: SharedPreferences? = null
    private var mDarkTheme = false

    override fun onCreate(savedInstanceState: Bundle?) {
        mSharedPrefs = PreferenceManager.getDefaultSharedPreferences(this)
        mDarkTheme = mSharedPrefs!!.getBoolean(SettingsFragment.PREFERENCE_THEME, false)

        if (mDarkTheme) setTheme(R.style.DarkTheme)

        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)



        mFoodListRecyclerView = findViewById(R.id.foodRecyclerView)
        mFoodDb = FoodDatabase.getInstance(applicationContext)

        val layoutManager: RecyclerView.LayoutManager = LinearLayoutManager(applicationContext)
        mFoodListRecyclerView!!.layoutManager = layoutManager
        mFoodAdapter = FoodAdapter(loadFoods())
        mFoodListRecyclerView!!.adapter = mFoodAdapter
    }

    override fun onResume() {
        super.onResume()

        // if theme is changed, change it here
        val darkTheme = mSharedPrefs!!.getBoolean(SettingsFragment.PREFERENCE_THEME, false)
        if (darkTheme != mDarkTheme) {
            recreate()
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        val inflater = menuInflater
        inflater.inflate(R.menu.subject_menu, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        // Handle item selection
        return when (item.itemId) {
            R.id.settings -> {
                val intent = Intent(this@MainActivity, SettingsActivity::class.java)
                startActivity(intent)
                true
            }
            R.id.about -> {
                val intent = Intent(this@MainActivity, AboutActivity::class.java)
                startActivity(intent)
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun onCourseEntered(food: Food) {
        if (food != null) {
            if (food.id == 0) {
                if (mFoodDb.addFood(food)) {
                    mFoodAdapter?.addCourse(food)
                } else {
                    val message = resources.getString("Already exists!".toInt(), food)
                    Toast.makeText(this@MainActivity, message, Toast.LENGTH_SHORT).show()
                }
            } else {
                if (food.id != 0) {
                    mFoodDb.updateFood(food)
                    mFoodAdapter?.updateCourse(food)
                } else {
                    // ??
                }
            }
        }
    }

    fun addFoodClick(view: View?) {
        mDialog = RecordDialogFragment()
        mDialog.getContext(FoodContext.ADD, mFoodDb)
        mDialog.show(supportFragmentManager, "")
    }

    private fun loadFoods() : MutableList<Food> {
        return mFoodDb.getFoods()
    }


    private inner class FoodHolder(inflater: LayoutInflater, parent: ViewGroup?) : RecyclerView.ViewHolder(inflater.inflate(R.layout.recycler_view_items, parent, false)), View.OnClickListener, View.OnLongClickListener {
        private var mFood: Food? = null
        private val mTextView: TextView

        fun bind(food: Food) {
            mFood = food
            mTextView.text = food.foodRestaurant
            mTextView.setBackgroundColor(Color.LTGRAY)
        }

        override fun onClick(view: View) {
            // TODO
            mDialog = FoodDialogFragment()
            mDialog.getContext(FoodContext.VIEW, mFoodDb)
            mDialog.getFoodId(mFood?.id!!)
            mDialog.show(supportFragmentManager, "")
        }

        override fun onLongClick(view: View?): Boolean {
            if (mActionMode != null) {
                return false
            }

            mSelectedFood = mFood!!
            mSelectedFoodPosition = adapterPosition

            // Re-bind the selected item
            mFoodAdapter?.notifyItemChanged(mSelectedFoodPosition)

            // Show the CAB
            mActionMode = this@MainActivity.startActionMode(mActionModeCallback)!!
            return true
        }

        private val mActionModeCallback: ActionMode.Callback = object : ActionMode.Callback {
            override fun onCreateActionMode(mode: ActionMode, menu: Menu?): Boolean {
                // Provide context menu for CAB
                val inflater = mode.menuInflater
                inflater.inflate(R.menu.context_menu, menu)
                return true
            }

            override fun onPrepareActionMode(mode: ActionMode?, menu: Menu?): Boolean {
                return false
            }

            override fun onActionItemClicked(mode: ActionMode, item: MenuItem): Boolean {
                // Process action item selection
                return when (item.itemId) {
                    R.id.delete -> {
                        // Delete from the database and remove from the RecyclerView
                        mFoodDb.deleteFood(mSelectedFood)
                        mFoodAdapter?.removeCourse(mSelectedFood)

                        // Close the CAB
                        mode.finish()
                        true
                    }
                    R.id.edit -> {
                        // TODO
                        // up comes a dialog box editing the desired list item
                        mFoodAdapter?.takeCourseIndex()
                        mDialog = FoodDialogFragment()
                        mDialog.getContext(FoodContext.EDIT, mFoodDb)
                        mDialog.getFoodId(mSelectedFood.id!!)
                        mDialog.show(supportFragmentManager, "")
                        true
                    }
                    else -> false
                }
            }

            override fun onDestroyActionMode(mode: ActionMode) {
                mActionMode = null

                // CAB closing, need to deselect item if not deleted
                mFoodAdapter?.notifyItemChanged(mSelectedFoodPosition)
                mSelectedFoodPosition = RecyclerView.NO_POSITION
            }
        }

        init {
            itemView.setOnClickListener(this)
            itemView.setOnLongClickListener(this)
            mTextView = itemView.findViewById(R.id.foodTextView)
        }
    }

    private inner class FoodAdapter(private val mCourseList: MutableList<Food>) : RecyclerView.Adapter<FoodHolder>() {
        private var mOldCourseIndex = 0

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): FoodHolder {
            val layoutInflater = LayoutInflater.from(applicationContext)
            return FoodHolder(layoutInflater, parent)
        }

        override fun onBindViewHolder(holder: FoodHolder, position: Int) {
            holder.bind(mCourseList[position])
        }

        override fun getItemCount(): Int {
            return mCourseList.size
        }

        fun takeCourseIndex() {
            val index = mSelectedFoodPosition
            mOldCourseIndex = index
        }

        fun removeCourse(course: Food?) {
            // Find subject in the list
            val index = mCourseList.indexOf(course)
            if (index >= 0) {
                // Remove the subject
                mCourseList.removeAt(index)

                // Notify adapter of subject removal
                notifyItemRemoved(index)
            }
        }

        fun addCourse(course: Food?) {
            // Add the new subject at the beginning of the list
            mCourseList.add(0, course!!)

            // Notify the adapter that item was added to the beginning of the list
            notifyItemInserted(0)

            // Scroll to the top
            mFoodListRecyclerView?.scrollToPosition(0)
        }

        fun updateCourse(course: Food?) {
            mCourseList.removeAt(mOldCourseIndex)
            mCourseList.add(0, course!!)
            notifyItemChanged(0)
            mFoodListRecyclerView?.scrollToPosition(0)
        }
    }
}